/* Copyright (C) 2009 Registro.br. All rights reserved. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 1. Redistribution of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
 */
package br.registro.dnsshim.domain;

import java.nio.ByteBuffer;
import java.util.Scanner;

import br.registro.dnsshim.util.DomainNameUtil;

public class Soa extends ResourceRecord {

	final private String mname;
	final private String rname;
	final private int serial;
	final private int refresh;
	final private int retry;
	final private int expire;
	final private int minimum;
	
	public Soa(String ownername, DnsClass dnsClass, int ttl,
			String mname, String rname,
			int serial, int refresh, int retry, int expire, int minimum) {
		super(ownername, RrType.SOA, dnsClass, ttl);
		this.mname = mname.toLowerCase();
		this.rname = rname.toLowerCase();
		this.serial = serial;
		this.refresh = refresh;
		this.retry = retry;
		this.expire = expire;
		this.minimum = minimum;
		this.rdata = RdataSoaBuilder.get(this.mname,	this.rname,
				serial, refresh, retry, expire, minimum);
	}
	
	public Soa(String ownername, DnsClass dnsClass, int ttl, String rdata) {
		super(ownername, RrType.SOA, dnsClass, ttl);
		Scanner scanner = new Scanner(rdata).useDelimiter("\\s+");
		mname = scanner.next().toLowerCase();
		rname = scanner.next().toLowerCase();
		
		String next = scanner.next();
		if (next.equals("(")) {
			serial = Integer.parseInt(scanner.next());
		} else {
			serial = Integer.parseInt(next);
		}
		
		refresh = Integer.parseInt(scanner.next());
		retry = Integer.parseInt(scanner.next());
		expire = Integer.parseInt(scanner.next());
		minimum = Integer.parseInt(scanner.next());
		
		this.rdata = RdataSoaBuilder.get(mname, rname,
				serial, refresh, retry, expire, minimum);
	}
	public String getMname() {
		return mname;
	}

	public String getRname() {
		return rname;
	}

	public int getSerial() {
		return serial;
	}

	public int getRefresh() {
		return refresh;
	}
	
	public int getRetry() {
		return retry;
	}

	public int getExpire() {
		return expire;
	}

	public int getMinimum() {
		return minimum;
	}
	
	@Override
	public String rdataPresentation() {
		StringBuilder rdata = new StringBuilder();
		rdata.append(mname).append(SEPARATOR);
		rdata.append(rname).append(SEPARATOR);
		rdata.append(serial).append(SEPARATOR);
		rdata.append(refresh).append(SEPARATOR);
		rdata.append(retry).append(SEPARATOR);
		rdata.append(expire).append(SEPARATOR);
		rdata.append(minimum);
	
		return rdata.toString();
	}

	public static Soa parseSoa(String ownername, DnsClass dnsClass, int ttl,
			ByteBuffer buffer) {
		buffer.getShort(); // rdlength
		String mname = DomainNameUtil.toPresentationFormat(buffer);
		String rname = DomainNameUtil.toPresentationFormat(buffer);
		int serial = buffer.getInt();
		int refresh = buffer.getInt();
		int retry = buffer.getInt();
		int expire = buffer.getInt();
		int minimum = buffer.getInt();
		
		return new Soa(ownername, dnsClass, ttl,
				mname, rname, serial, refresh, retry, expire, minimum);
	}
}
