/* Copyright (C) 2009 Registro.br. All rights reserved. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 1. Redistribution of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
 */
package br.registro.dnsshim.xfrd.domain;

import br.registro.dnsshim.common.server.ClientConfig;
import br.registro.dnsshim.common.server.ServerConfig;

public class XfrdConfig {
	private ServerConfig uiServerConfig = new ServerConfig();
	private ServerConfig dnsServerConfig = new ServerConfig();
	private ClientConfig signerClientConfig = new ClientConfig();

	public static final int DEFAULT_DNSSERVER_PORT = 53;
	public static final int DEFAULT_UI_PORT = 9999;
	public static final int DEFAULT_SIGNER_PORT = 9797;
	public static final String DEFAULT_SIGNER_HOST = "localhost";
	public static final String DEFAULT_CA_PATH = "";
	public static final String DEFAULT_CA_PASSWD = "";
	public static final int DEFAULT_TSIG_FUDGE = 300;
	public static final int DEFAULT_SCHEDULER_LOW_LIMIT = 10 * 24; // 10 days
	public static final int DEFAULT_SCHEDULER_HIGH_LIMIT = 5 * 24; // 5 days
	public static final int DEFAULT_SLAVE_SYNC_PERIOD = 30 * 60; // 30 minutes
	public static final String DEFAULT_SLAVESYNC_PATH = "./";
	public static final String DEFAULT_SLAVE_PATH = "";
	public static final String DEFAULT_SLAVE_USER = "";

	private short tsigFudge = DEFAULT_TSIG_FUDGE;
	private int schedulerLowLimit = DEFAULT_SCHEDULER_LOW_LIMIT; // low priority
	private int schedulerHighLimit = DEFAULT_SCHEDULER_HIGH_LIMIT; // high priority
	private int slaveSyncPeriod = DEFAULT_SLAVE_SYNC_PERIOD;
	private String slaveSyncPath = DEFAULT_SLAVESYNC_PATH;
	private String slavePath = DEFAULT_SLAVE_PATH;
	private boolean slaveSyncEnabled = true;
	private String slaveUser = DEFAULT_SLAVE_USER;

	public XfrdConfig() {
		uiServerConfig.setPort(DEFAULT_SIGNER_PORT);
		dnsServerConfig.setPort(DEFAULT_DNSSERVER_PORT);
		signerClientConfig.setHost(DEFAULT_SIGNER_HOST);
		signerClientConfig.setPort(DEFAULT_SIGNER_PORT);
	}
	
	public ServerConfig getUiServerConfig() {
		return uiServerConfig;
	}

	public void setUiServerConfig(ServerConfig uiServerConfig) {
		this.uiServerConfig = uiServerConfig;
	}

	public ServerConfig getDnsServerConfig() {
		return dnsServerConfig;
	}

	public void setDnsServerConfig(ServerConfig dnsServerConfig) {
		this.dnsServerConfig = dnsServerConfig;
	}

	public ClientConfig getSignerClientConfig() {
		return signerClientConfig;
	}

	public void setSignerClientConfig(ClientConfig signerClientConfig) {
		this.signerClientConfig = signerClientConfig;
	}

	public short getTsigFudge() {
		return tsigFudge;
	}

	public void setTsigFudge(short tsigFudge) {
		this.tsigFudge = tsigFudge;
	}

	public int getSchedulerLowLimit() {
		return schedulerLowLimit;
	}

	public void setSchedulerLowLimit(int schedulerLowLimit) {
		this.schedulerLowLimit = schedulerLowLimit;
	}

	public int getSchedulerHighLimit() {
		return schedulerHighLimit;
	}

	public void setSchedulerHighLimit(int schedulerHighLimit) {
		this.schedulerHighLimit = schedulerHighLimit;
	}

	public int getSlaveSyncPeriod() {
		return slaveSyncPeriod;
	}

	public void setSlaveSyncPeriod(int slaveSyncPeriod) {
		this.slaveSyncPeriod = slaveSyncPeriod;
	}

	public String getSlaveSyncPath() {
		return slaveSyncPath;
	}

	public void setSlaveSyncPath(String slaveSyncPath) {
		this.slaveSyncPath = slaveSyncPath;
	}

	public String getSlavePath() {
		return slavePath;
	}

	public void setSlavePath(String slavePath) {
		this.slavePath = slavePath;
	}

	public boolean isSlaveSyncEnabled() {
		return slaveSyncEnabled;
	}

	public void setSlaveSyncEnabled(boolean slaveSyncEnabled) {
		this.slaveSyncEnabled = slaveSyncEnabled;
	}

	public String getSlaveUser() {
		return slaveUser;
	}

	public void setSlaveUser(String slaveUser) {
		this.slaveUser = slaveUser;
	}
}