/* Copyright (C) 2009 Registro.br. All rights reserved. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 1. Redistribution of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
 */
package br.registro.dnsshim.xfrd.service;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.Map.Entry;

import org.apache.log4j.Logger;

import br.registro.dnsshim.common.server.ClientConfig;
import br.registro.dnsshim.common.server.DnsshimProtocolException;
import br.registro.dnsshim.common.server.ProtocolStatusCode;
import br.registro.dnsshim.domain.Dnskey;
import br.registro.dnsshim.domain.DnskeyFlags;
import br.registro.dnsshim.domain.DnskeyProtocol;
import br.registro.dnsshim.domain.DnskeyType;
import br.registro.dnsshim.domain.DsDigestType;
import br.registro.dnsshim.domain.DsInfo;
import br.registro.dnsshim.domain.ResourceRecord;
import br.registro.dnsshim.domain.Soa;
import br.registro.dnsshim.domain.Zone;
import br.registro.dnsshim.domain.logic.DsCalc;
import br.registro.dnsshim.signer.domain.logic.DnskeyCalc;
import br.registro.dnsshim.signer.protocol.RemoveKeyRequest;
import br.registro.dnsshim.util.ByteUtil;
import br.registro.dnsshim.xfrd.dao.filesystem.DnskeyInfoDao;
import br.registro.dnsshim.xfrd.dao.filesystem.ZoneDataDao;
import br.registro.dnsshim.xfrd.dao.filesystem.ZoneDataPrePublishDao;
import br.registro.dnsshim.xfrd.dao.filesystem.ZoneInfoDao;
import br.registro.dnsshim.xfrd.dao.filesystem.ZonenameDao;
import br.registro.dnsshim.xfrd.dns.axfr.AxfrClient;
import br.registro.dnsshim.xfrd.dns.notifier.domain.ZoneNotify;
import br.registro.dnsshim.xfrd.dns.notifier.logic.NotifierManager;
import br.registro.dnsshim.xfrd.domain.DnskeyInfo;
import br.registro.dnsshim.xfrd.domain.DnskeyStatus;
import br.registro.dnsshim.xfrd.domain.Slave;
import br.registro.dnsshim.xfrd.domain.SlaveGroup;
import br.registro.dnsshim.xfrd.domain.User;
import br.registro.dnsshim.xfrd.domain.XfrdConfig;
import br.registro.dnsshim.xfrd.domain.ZoneData;
import br.registro.dnsshim.xfrd.domain.ZoneDataPrePublish;
import br.registro.dnsshim.xfrd.domain.ZoneInfo;
import br.registro.dnsshim.xfrd.domain.Zonename;
import br.registro.dnsshim.xfrd.domain.logic.DnskeyInfoCache;
import br.registro.dnsshim.xfrd.domain.logic.DnsshimSession;
import br.registro.dnsshim.xfrd.domain.logic.DnsshimSessionCache;
import br.registro.dnsshim.xfrd.domain.logic.SlaveGroupCache;
import br.registro.dnsshim.xfrd.domain.logic.UserCache;
import br.registro.dnsshim.xfrd.domain.logic.XfrdConfigManager;
import br.registro.dnsshim.xfrd.domain.logic.ZoneDataCache;
import br.registro.dnsshim.xfrd.domain.logic.ZoneDataPrePublishCache;
import br.registro.dnsshim.xfrd.domain.logic.ZoneInfoCache;
import br.registro.dnsshim.xfrd.domain.logic.ZonenameCache;
import br.registro.dnsshim.xfrd.scheduler.ResignManager;
import br.registro.dnsshim.xfrd.signerclient.SignerClient;
import br.registro.dnsshim.xfrd.ui.protocol.ChangeKeyStatusResponse;
import br.registro.dnsshim.xfrd.ui.protocol.ImportZoneRequest;
import br.registro.dnsshim.xfrd.ui.protocol.ImportZoneResponse;
import br.registro.dnsshim.xfrd.ui.protocol.ListSlavesResponse;
import br.registro.dnsshim.xfrd.ui.protocol.ListZonesBySlaveGroupResponse;
import br.registro.dnsshim.xfrd.ui.protocol.ListZonesRequest;
import br.registro.dnsshim.xfrd.ui.protocol.ListZonesResponse;
import br.registro.dnsshim.xfrd.ui.protocol.NewKeyRequest;
import br.registro.dnsshim.xfrd.ui.protocol.NewKeyResponse;
import br.registro.dnsshim.xfrd.ui.protocol.NewZoneRequest;
import br.registro.dnsshim.xfrd.ui.protocol.NewZoneResponse;
import br.registro.dnsshim.xfrd.ui.protocol.PrintZoneResponse;
import br.registro.dnsshim.xfrd.ui.protocol.RemoveKeyResponse;
import br.registro.dnsshim.xfrd.ui.protocol.RemoveZoneResponse;
import br.registro.dnsshim.xfrd.ui.protocol.RrResponse;
import br.registro.dnsshim.xfrd.ui.protocol.SetExpirationPeriodRequest;
import br.registro.dnsshim.xfrd.ui.protocol.SetExpirationPeriodResponse;
import br.registro.dnsshim.xfrd.ui.protocol.SlaveGroupRequest;
import br.registro.dnsshim.xfrd.ui.protocol.SlaveResponse;
import br.registro.dnsshim.xfrd.ui.protocol.ZoneUserRequest;
import br.registro.dnsshim.xfrd.ui.protocol.ZoneUserResponse;
import br.registro.dnsshim.xfrd.ui.protocol.ZoneVersionResponse;

public class ZoneServiceImpl implements ZoneService {
	private static final Logger logger = Logger.getLogger(ZoneServiceImpl.class);

	private ZoneDataCache zoneDataCache = ZoneDataCache.getInstance();
	private ZoneDataPrePublishCache zoneDataPrePubCache = ZoneDataPrePublishCache.getInstance();
	private ZoneInfoCache zoneInfoCache = ZoneInfoCache.getInstance();
	private ZonenameCache zonenameCache = ZonenameCache.getInstance();

	
	@Override
	public NewZoneResponse newZone(NewZoneRequest request)
		throws DnsshimProtocolException, IOException {
		// Check if the zone already exists
		String zonename = request.getZone();
		
		NewZoneResponse response = new NewZoneResponse();
		response.setZone(zonename);
		
		ZoneDataCache zoneDataCache = ZoneDataCache.getInstance();
		ZoneDataPrePublishCache zoneDataPrePubCache = ZoneDataPrePublishCache.getInstance();

		if (zoneDataPrePubCache.get(zonename)!= null ||
				zoneDataCache.get(zonename) != null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_ALREADY_EXISTS, "Zone already exists");
		}
		
		Soa soa = request.getSoa();
		ZoneInfo zoneInfo = new ZoneInfo();
		zoneInfo.setZonename(zonename);
		zoneInfo.setSoaMinimum(soa.getMinimum());
		zoneInfo.setTtlZone(soa.getTtl());
		zoneInfo.setTtlDnskey(soa.getTtl());
		zoneInfo.setExpiration(request.getExpirationPeriod());
		zoneInfo.setSigned(request.isSigned());
		
		DnsshimSessionCache dnsshimSessionCache = DnsshimSessionCache.getInstance();
		DnsshimSession session = dnsshimSessionCache.get(request.getSessionId());
		String username = (String) session.getAttribute("username");
		if (username != null) {
			zoneInfo.addUser(username);
		}
		
		zoneInfoCache.put(zoneInfo);

		Zone zone = new Zone(zonename, zoneInfo.isSigned());
		zone.add(soa);
		
		ZoneDataPrePublish zoneDataPre = new ZoneDataPrePublish(zone);
		zoneDataPrePubCache.put(zoneDataPre);

		zonenameCache.put(new Zonename(zonename));
		
		if (request.isSigned()) {
			NewKeyRequest keyRequest = new NewKeyRequest();
			keyRequest.setZone(zonename);
			keyRequest.setAlgorithm(request.getAlgorithm());
			keyRequest.setKeySize(request.getKeySize());
			keyRequest.setKeyStatus(DnskeyStatus.SIGN);
			
			// The key being created with the zone must be a ZSK
			keyRequest.setKeyType(DnskeyType.ZSK);
			keyRequest.setFlags(DnskeyFlags.SEP_ON);
			keyRequest.setProtocol(DnskeyProtocol.PROTOCOL);
			KeyServiceImpl keyService = new KeyServiceImpl();
			NewKeyResponse keyResponse = keyService.newKey(keyRequest);
			
			response.setSigned(true);
			response.setKeyName(keyResponse.getKeyName());
			response.setDsInfo(keyResponse.getDsInfo());
		}
		
		if (logger.isInfoEnabled()) {
			logger.info("Zone " + zonename + " created.");
		}
		return response;
	}

	@Override
	public ImportZoneResponse importZone(ImportZoneRequest request)
			throws DnsshimProtocolException, IOException {
		// Check if the zone already exists
		String zonename = request.getZone().toLowerCase();

		if (zoneDataPrePubCache.get(zonename)!= null ||
				zoneDataCache.get(zonename) != null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_ALREADY_EXISTS, "Zone already exists");
		}

		String server = request.getServer();
		int port = request.getPort();
		AxfrClient axfrClient = new AxfrClient(server, port);
		axfrClient.requestAxfr(zonename, request.getSessionId());
		zonenameCache.put(new Zonename(zonename));
		
		ImportZoneResponse response = new ImportZoneResponse();
		response.setZone(zonename);
		response.setMsg("Zone imported successfully");

		if (logger.isInfoEnabled()) {
			logger.info("Zone " + zonename + " imported from " + server);
		}
		return response;
	}

	@Override
	public RrResponse addRecord(String zonename, ResourceRecord rr)
		throws DnsshimProtocolException, IOException {
		RrResponse response = new RrResponse();

		ZoneDataPrePublish zoneData = zoneDataPrePubCache.get(zonename);
		if (zoneData == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_NOT_FOUND, "Zone not found");
		}
		
		if (!zoneData.add(rr)) {
			throw new DnsshimProtocolException(ProtocolStatusCode.RESOURCE_RECORD_ADD_ERROR, "RR add error");
		}

		zoneDataPrePubCache.put(zoneData);

		if (logger.isInfoEnabled()) {
			logger.info("Record " + rr.toString() + " added to zone " + zonename);
		}
		return response;
	}
	
	@Override
	public RrResponse removeRecord(String zonename, ResourceRecord rr)
		throws IOException, DnsshimProtocolException {
		RrResponse response = new RrResponse();
		ZoneDataPrePublish zoneData = zoneDataPrePubCache.get(zonename);
		if (zoneData == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_NOT_FOUND, "Zone not found");
		}
		
		if (zoneData.remove(rr) == false) {
			throw new DnsshimProtocolException(ProtocolStatusCode.RESOURCE_RECORD_NOT_FOUND, "Record not found in zone");
		}
		
		zoneDataPrePubCache.put(zoneData);
		
		if (logger.isInfoEnabled()) {
			logger.info("Record " + rr.toString() + " removed from zone " + zonename);
		}
		return response;
	}
	
	public ListZonesResponse listZones(ListZonesRequest request) throws IOException, DnsshimProtocolException {
		ListZonesResponse response = new ListZonesResponse();
		DnsshimSessionCache dnsshimSessionCache = DnsshimSessionCache.getInstance();
		DnsshimSession session = dnsshimSessionCache.get(request.getSessionId());
		String username = (String) session.getAttribute("username");
		for (String zonename : zonenameCache.keySet()) {
			ZoneInfo zoneInfo = zoneInfoCache.get(zonename);
			if (zoneInfo.getUsers().contains(username)) {
				response.addZonename(new Zonename(zonename));
			}	
		}
		return response;
	}

	public ListZonesBySlaveGroupResponse listZonesBySlaveGroup(String slaveGroupName) {
		ListZonesBySlaveGroupResponse response = new ListZonesBySlaveGroupResponse();
		SlaveGroupCache slaveGroupCache = SlaveGroupCache.getInstance();
		SlaveGroup slaveGroup = slaveGroupCache.get(slaveGroupName); 
		for (Zonename zonename : slaveGroup.getZonenames()) {
			response.addZonename(zonename);
		}
		response.setSlaveGroup(slaveGroupName);
		
		return response;
	}
	
	public PrintZoneResponse printZone(String zonename)
	throws DnsshimProtocolException, IOException {
		PrintZoneResponse response = new PrintZoneResponse();
		response.setZone(zonename);
		ZoneDataPrePublish zoneData = zoneDataPrePubCache.get(zonename);
		if (zoneData == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_NOT_FOUND, "Zone not found");
		}

		Iterator<ResourceRecord> it = zoneData.getAxfr().iterator();
		while (it.hasNext()) {
			ResourceRecord rr = it.next();
			if (rr.getType().isDnssec()) {
				continue;
			}
			response.addRr(rr);
		}
		
		DnskeyInfoCache dnskeyInfoCache = DnskeyInfoCache.getInstance();
		List<Dnskey> dnskeys = new ArrayList<Dnskey>();
		ZoneInfo zoneInfo = zoneInfoCache.get(zonename);
		for (String keyName : zoneInfo.getKeys(DnskeyStatus.PUBLISH)) {
			DnskeyInfo dnskeyInfo = dnskeyInfoCache.get(keyName);
			Dnskey dnskey = dnskeyInfo.getDnskey();
			dnskeys.add(dnskey);
		}
		
		for (String keyName : zoneInfo.getKeys(DnskeyStatus.SIGN)) {
			DnskeyInfo dnskeyInfo = dnskeyInfoCache.get(keyName);
			Dnskey dnskey = dnskeyInfo.getDnskey();
			dnskeys.add(dnskey);
		}

		for (Dnskey dnskey : dnskeys) {
			String digest = "";
			try {
				digest = DsCalc.calcDigest(dnskey, DsDigestType.SHA1, zonename);
			} catch (NoSuchAlgorithmException nsae) {
				
			}
			short keytag = DnskeyCalc.calcKeyTag(dnskey.getRdata());
			DsInfo dsInfo = new DsInfo();
			dsInfo.setKeytag(ByteUtil.toUnsigned(keytag));
			dsInfo.setDigestType(DsDigestType.SHA1);
			dsInfo.setDigest(digest);
			response.addDs(dsInfo);
		}
		return response;
	}
	
	public RemoveZoneResponse removeZone(String zone) throws DnsshimProtocolException, IOException {
		RemoveZoneResponse response = new RemoveZoneResponse();
		response.setZone(zone);
		
		Zonename zonename = zonenameCache.get(zone);		
		ZoneInfo zoneInfo = zoneInfoCache.get(zone);
		ZoneDataPrePublish zonePrePublish = zoneDataPrePubCache.get(zone);
		if (zonename == null ||
				zoneInfo == null ||
				zonePrePublish == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_NOT_FOUND, "Zone not found");
		}
		
		List<String> zoneKeys = zoneInfo.getAllKeys();
		for (String key : zoneKeys) {
			removeKey(zone, key);
		}

		List<String> slaves = zoneInfo.getSlaveGroups();
		SlaveGroupCache slaveGroupCache = SlaveGroupCache.getInstance();
		for (String slave : slaves) {
			SlaveGroup slaveGroup = slaveGroupCache.get(slave);
			slaveGroup.removeZonename(zonename);
			slaveGroupCache.put(slaveGroup);
		}
		
		ResignManager.remove(zone);
		zonenameCache.remove(zone);
		zoneInfoCache.remove(zone);
		zoneDataPrePubCache.remove(zone);
		
		ZoneData zoneData = zoneDataCache.get(zone);
		if (zoneData != null) {
			zoneDataCache.remove(zone);
		}

		ZonenameDao zonenameDao = new ZonenameDao();
		zonenameDao.refresh(zonenameCache.keySet());
		ZoneDataDao zoneDataDao = new ZoneDataDao();
		zoneDataDao.delete(zone);
		ZoneDataPrePublishDao zonePrePublishDao = new ZoneDataPrePublishDao();
		zonePrePublishDao.delete(zone);
		ZoneInfoDao zoneInfoDao = new ZoneInfoDao();
		zoneInfoDao.delete(zone);
		
		if (logger.isInfoEnabled()) {
			logger.info("Zone " + zonename + " removed.");
		}
		return response;
	}
	
	public ZoneVersionResponse zoneVersion(String zonename)
		throws DnsshimProtocolException, IOException {
		ZoneDataPrePublish zoneData = zoneDataPrePubCache.get(zonename);
		if (zoneData == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_NOT_FOUND, "Zone not found");
		}

		ZoneVersionResponse response = new ZoneVersionResponse();
		Zone zone = zoneData.getAxfr();
		response.setZone(zonename);
		response.setSerial(zone.getSerial());
		return response;
	}

	public SlaveResponse assignSlaveGroup(SlaveGroupRequest request) 
	throws DnsshimProtocolException, IOException {
		SlaveGroupCache slaveCache = SlaveGroupCache.getInstance();
		
		String groupName = request.getGroupName();
		
		Zonename zonename = zonenameCache.get(request.getZone());
		SlaveGroup slaveGroup = slaveCache.get(groupName);
		
		if (slaveGroup == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.SLAVE_GROUP_NOT_FOUND, "Slave group not found");
		}
		ZoneInfo zoneInfo = zoneInfoCache.get(zonename.getName());
		if (zoneInfo == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_NOT_FOUND, "Zone not found");
		}
		
		List<String> slaves = zoneInfo.getSlaveGroups();
		if (slaves.contains(groupName)) {
			throw new DnsshimProtocolException(ProtocolStatusCode.SLAVE_ALREADY_EXISTS, "Slave already in zone");
		}
		
		zoneInfo.assignSlaveGroup(groupName);
		zoneInfoCache.put(zoneInfo);
		if (slaveGroup.addZonename(zonename)){ 
			slaveCache.put(slaveGroup);
		}
				
		// If the zone is published, notify the new slave
		ZoneData zoneData = zoneDataCache.get(zonename.getName());

		if (zoneData != null) {
			NotifierManager.clear(zonename.getName());
			for (Entry<String, Slave> entry : slaveGroup.getSlaves().entrySet()) {
				Slave s = entry.getValue();
				ZoneNotify notify = new ZoneNotify();
				notify.setZonename(zonename.getName());
				notify.setSlaveAddress(s.getAddress().getHostAddress());
				notify.setSlavePort(s.getPort());
				notify.setNextNotify(new Date());
				Random random = new Random();
				int id = random.nextInt(65534) + 1;
				notify.setQueryId((short) id);
				NotifierManager.addZoneNotify(notify);	
			}
		}
		
		if (logger.isInfoEnabled()) {
			logger.info("Slavegroup " + groupName + " assigned to zone " + zonename);
		}
		SlaveResponse response = new SlaveResponse();
		return response;
	}

	@Override
	public SlaveResponse unassignSlaveGroup(SlaveGroupRequest request)
			throws DnsshimProtocolException, IOException {
		SlaveGroupCache slaveGroupCache = SlaveGroupCache.getInstance();

		Zonename zonename = zonenameCache.get(request.getZone());
		String groupName = request.getGroupName();
		ZoneInfo zoneInfo = zoneInfoCache.get(zonename.getName());
		if (zoneInfo == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_NOT_FOUND, "Zone not found");
		}
		
		if (zoneInfo.unassignSlaveGroup(groupName) == false) {
			throw new DnsshimProtocolException(ProtocolStatusCode.SLAVE_NOT_FOUND, "Slave not in zone");
		}		
	
		zoneInfoCache.put(zoneInfo);
		
		SlaveGroup slaveGroup = slaveGroupCache.get(groupName);
		slaveGroup.removeZonename(zonename);
		slaveGroupCache.put(slaveGroup);
		
		if (logger.isInfoEnabled()) {
			logger.info("Slavegroup " + groupName + " unassigned from zone " + zonename);
		}
		SlaveResponse response = new SlaveResponse();
		return response;
	}

	public ListSlavesResponse listSlaves(String zonename)
			throws DnsshimProtocolException, IOException {
		ZoneInfo zoneInfo = zoneInfoCache.get(zonename);
		if (zoneInfo == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_NOT_FOUND, "Zone not found");
		}

		List<String> slaveGroups = zoneInfo.getSlaveGroups();
		ListSlavesResponse response = new ListSlavesResponse();
		response.setZone(zonename);
		response.setSlaveGroups(slaveGroups);
		return response;
	}
	
	@Override
	public RemoveKeyResponse removeKey(String zonename, String keyName) throws IOException, DnsshimProtocolException {
		RemoveKeyResponse response = new RemoveKeyResponse();

		ZoneInfo zoneInfo = zoneInfoCache.get(zonename);
		
		if (zoneInfo == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_NOT_FOUND, "Zone not found");
		}
		
		DnskeyStatus keyStatus = zoneInfo.hasKey(keyName);
		if (keyStatus == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.INVALID_KEY, "Key not found");
		}
		
		// Remove the key from the zone
		zoneInfo.removeKey(keyName, keyStatus);
		DnskeyInfoCache dnskeyInfoCache = DnskeyInfoCache.getInstance();
		dnskeyInfoCache.remove(keyName);
		
		// Remove the key files
		DnskeyInfoDao dao = new DnskeyInfoDao();
		dao.delete(keyName);
		XfrdConfig config = XfrdConfigManager.getInstance();
		ClientConfig signerConfig = config.getSignerClientConfig();
		SignerClient client = new SignerClient(signerConfig.getHost(),
				signerConfig.getPort(), signerConfig.getCaPath(), signerConfig.getCaPasswd());
		RemoveKeyRequest signerRequest = new RemoveKeyRequest();
		signerRequest.setKeyName(keyName);
		client.removeKey(signerRequest);

		// Check if the zone has any signing keys
		if (zoneInfo.getKeys(DnskeyStatus.SIGN).isEmpty()) {
			ZoneDataPrePublish zoneData = zoneDataPrePubCache.get(zonename);
			Zone zone = zoneData.getAxfr();
			zone.setSigned(false);
			zoneInfo.setSigned(false);

			zoneDataPrePubCache.put(zoneData);
		}
		
		zoneInfoCache.put(zoneInfo);

		if (logger.isInfoEnabled()) {
			logger.info("Key " + keyName + " removed from zone " + zonename);
		}
		return response;
	}
	
	@Override
	public ChangeKeyStatusResponse changeKey(String zonename, String keyName, DnskeyStatus oldStatus, DnskeyStatus newStatus) throws DnsshimProtocolException, IOException {
		ChangeKeyStatusResponse response = new ChangeKeyStatusResponse();
		DnskeyInfoCache dnskeyInfoCache = DnskeyInfoCache.getInstance();

		ZoneInfo zoneInfo = zoneInfoCache.get(zonename);
		if (zoneInfo == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_NOT_FOUND, "Zone not found");
		}
		
		DnskeyStatus keyStatus = zoneInfo.hasKey(keyName);
		if (keyStatus == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.INVALID_KEY, "Key not found");
		}
		
		if (keyStatus != oldStatus) {
			throw new DnsshimProtocolException(ProtocolStatusCode.INVALID_KEY_STATUS, "Incompatible key status");
		}
		
		zoneInfo.removeKey(keyName, oldStatus);
		zoneInfo.addKey(keyName, newStatus);
		
		try {
			DnskeyInfo dnskeyInfo = dnskeyInfoCache.get(keyName); 
			Dnskey dnskey = dnskeyInfo.getDnskey();
			dnskey.setTtl(zoneInfo.getTtlZone());
			
			String digest = DsCalc.calcDigest(dnskey, DsDigestType.SHA1, zonename);
			short keytag = DnskeyCalc.calcKeyTag(dnskey.getRdata());
			DsInfo dsInfo = new DsInfo();
			dsInfo.setKeytag(ByteUtil.toUnsigned(keytag));
			dsInfo.setDigestType(DsDigestType.SHA1);
			dsInfo.setDigest(digest);
			response.setDsInfo(dsInfo);
			
			// Check if the zone has any signing keys
			ZoneDataPrePublish zoneData = zoneDataPrePubCache.get(zonename);
			Zone zone = zoneData.getAxfr();
			if (zoneInfo.getKeys(DnskeyStatus.SIGN).isEmpty()) {
				zone.setSigned(false);
				zoneInfo.setSigned(false);
			} else {
				zone.setSigned(true);
				zoneInfo.setSigned(true);
			}
			zoneDataPrePubCache.put(zoneData);
			zoneInfoCache.put(zoneInfo);
		} catch (NoSuchAlgorithmException nsae) {
			throw new DnsshimProtocolException(ProtocolStatusCode.INVALID_ALGORITHM, nsae.getMessage());
		}
		
		if (logger.isInfoEnabled()) {
			logger.info("Key status changed to " + newStatus.toString() +
					" on key " + keyName + "(" + zonename + ")");
		}
		return response;
	}

	@Override
	public SetExpirationPeriodResponse setExpirationPeriod(SetExpirationPeriodRequest request)
			throws DnsshimProtocolException {
		String zonename = request.getZone();
		int expiration = request.getExpirationPeriod();
		
		ZoneInfo zoneInfo = zoneInfoCache.get(zonename);
		if (zoneInfo == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_NOT_FOUND, "Zone not found");
		}
		
		zoneInfo.setExpiration(expiration);
		zoneInfoCache.put(zoneInfo);
		
		if (logger.isInfoEnabled()) {
			logger.info("Expiration period set to " + String.valueOf(zoneInfo.getExpiration()) +
					" on zone " + zonename);
		}
		SetExpirationPeriodResponse response = new SetExpirationPeriodResponse();
		response.setZone(zonename);
		response.setExpirationPeriod(zoneInfo.getExpiration());
		return response;
	}

	@Override
	public ZoneUserResponse addUser(ZoneUserRequest request)
			throws DnsshimProtocolException {
		String zonename = request.getZone();
		String username = request.getUsername();
		
		UserCache userCache = UserCache.getInstance();
		User user = userCache.get(username);
		 
		if (user == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.USER_NOT_FOUND, "User not found");
		}
		
		ZoneInfo zoneInfo = zoneInfoCache.get(zonename);
		if (zoneInfo == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_NOT_FOUND, "Zone not found");
		}
		
		if (!zoneInfo.getUsers().contains(username)) {
			zoneInfo.addUser(request.getUsername());
			zoneInfoCache.put(zoneInfo);
		} else {
			throw new DnsshimProtocolException(ProtocolStatusCode.REFUSED_OPERATION, "User already owns this zone");
		}
		
		if (logger.isInfoEnabled()) {
			logger.info("User " + username + " added to zone " + zonename);
		}
		return new ZoneUserResponse();
	}

	@Override
	public ZoneUserResponse removeUser(ZoneUserRequest request)
			throws DnsshimProtocolException {
		
		String zonename = request.getZone();
		String username = request.getUsername();
		
		UserCache userCache = UserCache.getInstance();
		User user = userCache.get(username);
		 
		if (user == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.USER_NOT_FOUND, "User not found");
		}
		
		ZoneInfo zoneInfo = zoneInfoCache.get(zonename);
		if (zoneInfo == null) {
			throw new DnsshimProtocolException(ProtocolStatusCode.ZONE_NOT_FOUND, "Zone not found");
		}
		
		if (zoneInfo.getUsers().contains(username)) {
			if (zoneInfo.getUsers().size() == 1) {
				throw new DnsshimProtocolException(ProtocolStatusCode.REFUSED_OPERATION, "Zone must have at least one user");
			} else {
				zoneInfo.getUsers().remove(username);
				zoneInfoCache.put(zoneInfo);
			}
		} else {
			throw new DnsshimProtocolException(ProtocolStatusCode.REFUSED_OPERATION, "User not owns this zone");
		}

		if (logger.isInfoEnabled()) {
			logger.info("User " + username + " removed from zone " + zonename);
		}
		return new ZoneUserResponse();
	}
}